"""Collection of different customised loss functions."""

from keras import backend as K

from typing import Callable


def l_p_loss(power: int) -> Callable:
    """
    Calculate the L<p> loss for given power p.

    L1 (p=1) is equal to mean absolute error (MAE), L2 (p=2) is to mean squared error (MSE), ...

    :param power: set the power of the error calculus

    :return: loss for given power
    """

    def l_p_loss(y_true, y_pred):
        return K.mean(K.pow(K.abs(y_pred - y_true), power), axis=-1)

    return l_p_loss


def var_loss(y_true, y_pred) -> Callable:
    return K.mean(K.square(K.var(y_true) - K.var(y_pred)))


def custom_loss(loss_list, loss_weights=None) -> Callable:
    n = len(loss_list)
    if loss_weights is None:
        loss_weights = [1. / n for _ in range(n)]
    else:
        assert len(loss_weights) == n
        loss_weights = [w / sum(loss_weights) for w in loss_weights]

    def loss(y_true, y_pred):
        return sum([loss_weights[i] * loss_list[i](y_true, y_pred) for i in range(n)])

    return loss
